/**
 * Created by zack on 2018/8/15.
 */

var SlideLayer = cc.Layer.extend({
    showSize: null,
    ctor: function (showSize, cList) {
        this._super();
        this.initData(showSize, cList);
        this.initUI();
        this.initListener();
        return true;
    },
    initData: function (showSize, cList) {
        this.cList = cList;
        this.showSize = showSize;
        this.itemNum = cList.length;
        this.tBP = null;
        this.originIndex = 1; // 初始下标
        this.itemList = [];
        this.normalScale = 0.6;
        this.selectScale = 1.0;
        this.dScale = this.selectScale - this.normalScale;
        this.scaleTime = 0.2;
        // 移动后的回调
        this.moveCallFunc = null;

    },
    initUI: function () {

        var baseLayout = new ccui.Layout();
        baseLayout.setContentSize(this.showSize);
        baseLayout.setBackGroundColor(cc.color.RED);
        baseLayout.setBackGroundColorType(ccui.Layout.BG_COLOR_SOLID);
        baseLayout.setBackGroundColorOpacity(0);
        baseLayout.setAnchorPoint(0.5, 0.5);
        baseLayout.setClippingEnabled(true);
        this.addChild(baseLayout);
        baseLayout.setTouchEnabled(true);
        baseLayout.setSwallowTouches(false);
        baseLayout.addTouchEventListener(function (sender, type) {
            if (type == ccui.Widget.TOUCH_BEGAN) {
                cc.log("showLayout TOUCH_BEGAN");
                this.isTouchBegan = true;
            }
        }, this);

        // 算出每个大小
        this.itemSize = cc.size(this.showSize.width / 3, this.showSize.height);
        this.originP = cc.p(this.itemSize.width / 2, this.itemSize.height / 2);
        // 创建底
        var showLayout = new ccui.Layout();
        showLayout.setAnchorPoint(0.5, 0.5);
        showLayout.setContentSize(this.showSize);
        showLayout.setBackGroundColor(cc.color.GREEN);
        showLayout.setBackGroundColorType(ccui.Layout.BG_COLOR_SOLID);
        showLayout.setBackGroundColorOpacity(0);
        baseLayout.addChild(showLayout);
        showLayout.setPosition(cc.p(this.showSize.width / 2, this.showSize.height / 2));
        //showLayout.setPositionX(this.itemSize.width * this.originIndex);

        this.showLayout = showLayout;
        this.showLayoutOP = showLayout.getPosition();


        for (var i = 0; i < this.itemNum; i++) {
            var itemLayout = new ccui.Layout();
            itemLayout.setContentSize(this.itemSize);
            itemLayout.setAnchorPoint(0, 0);
            itemLayout.setBackGroundColor(cc.color.BLUE);
            itemLayout.setBackGroundColorType(ccui.Layout.BG_COLOR_SOLID);
            itemLayout.setBackGroundColorOpacity(0);
            showLayout.addChild(itemLayout);
            itemLayout.setTouchEnabled(false);
            itemLayout.setPosition(cc.p(this.originP.x + this.itemSize.width * i, this.originP.y));
            itemLayout.index = i;
            this.itemList.push(itemLayout);
            //var label = new cc.LabelTTF("" + i, "", 30);
            //label.setColor(Config.BK_COLOR);
            //itemLayout.addChild(label, 30);

            itemLayout.addChild(this.cList[i]);

        }
        // this.moveToIndex(1);
        this.initCenter();
        // this.moveToIndex(this.centerIndex);
    },
    initCenter: function () {
        // 先放在中间
        var cp = parseInt(this.itemNum / 2);
        this.showLayout.x = this.showLayout.x - (cp - 1) * this.itemSize.width;
        this.selectIndex = cp;
        this.showOriginPx = this.showLayout.x;
        this.centerIndex = cp;
    },

    moveToIndex: function (index, isNoTime) {

        var time = this.scaleTime;
        if (isNoTime) time = 0;

        var dx = index - this.selectIndex;
        cc.log("moveToIndex dx : " + dx);
        cc.log("this.selectIndex  : " + this.selectIndex);

        // this.showLayout.x = this.showLayout.x - dx * this.itemSize.width;
        this.selectIndex = index;
        var endX = this.showOriginPx - dx * this.itemSize.width;

        // 移动
        var lastX = this.itemList[this.itemList.length - 1].x;
        var firstX = this.itemList[0].x;

        // 向左移动,添加到右最后
        if (dx >= 0) {
            for (var i = 1; i <= dx; i++) {
                var item = this.itemList.shift();
                item.x = lastX + i * this.itemSize.width;
                this.itemList.push(item);
            }
        } else {
            dx = Math.abs(dx);
            for (var i = 1; i <= dx; i++) {
                var item = this.itemList.pop();
                item.x = firstX - i * this.itemSize.width;
                this.itemList.unshift(item);
            }
        }


        this.showLayout.stopAllActions();
        this.isMoveing = true;
        var action = cc.moveTo(time, cc.p(endX, this.showLayoutOP.y));

        this.showLayout.runAction(cc.sequence(action, cc.callFunc(function () {
            this.isMoveing = false;
        }.bind(this), this)));

        for (var i = 0; i < this.itemList.length; i++) {
            var item = this.itemList[i];
            item.stopAllActions();
            if (this.centerIndex == i) {
                item.runAction(cc.scaleTo(time, this.selectScale));
            } else {
                item.runAction(cc.scaleTo(time, this.normalScale));
            }
        }

        this.trueSlectIndex = this.itemList[this.centerIndex].index;
        if (this.moveCallFunc) this.moveCallFunc(this.trueSlectIndex);

    },
    getTrueIndex: function (index) {
        var absIndex = Math.abs(index);
        var trueIndex = absIndex - parseInt(absIndex / this.itemNum) * this.itemNum;

        return trueIndex;
    },
    alignPanel: function () {
        var dx = this.showLayout.x - this.showOriginPx;
        var fNum = dx / this.itemSize.width;
        var num = Math.round(fNum);
        cc.log("alignPanel num: " + num);
        //this.showLayout.x = this.showOriginPx;
        this.moveToIndex(this.selectIndex - num);
    },
    initListener: function () {
        cc.eventManager.addListener({
            event: cc.EventListener.TOUCH_ONE_BY_ONE,
            swallowTouches: true,
            onTouchBegan: this.onTouchBegan,
            onTouchMoved: this.onTouchMoved,
            onTouchEnded: this.onTouchEnded,
            onTouchCancelled: this.onTouchCancelled
        }, this);
    },
    onTouchBegan: function (touch, event) {
        cc.log("onTouchBegan");
        var tag = event.getCurrentTarget();
        if (tag.isMoveing) return false;

        tag.tBP = touch.getLocation();
        if (!tag.isTouchBegan) return false;
        tag.showOriginPx = tag.showLayout.x;
        return true;
    },
    onTouchMoved: function (touch, event) {
        cc.log("onTouchMoved");
        var tag = event.getCurrentTarget();
        var endP = touch.getLocation();
        var dx = endP.x - tag.tBP.x;
        tag.tBP = endP;
        tag.showLayout.x = tag.showLayout.x + dx;
        tag.moveScale(dx);
    },
    onTouchEnded: function (touch, event) {
        cc.log("onTouchEnded");
        var tag = event.getCurrentTarget();
        tag.isTouchBegan = false;
        tag.alignPanel();
    },
    onTouchCancelled: function (touch, event) {
        cc.log("onTouchCancelled");
        var tag = event.getCurrentTarget();
        tag.isTouchBegan = false;
        tag.alignPanel();
    },
    leftMove: function () {
        if (this.isMoveing) return;
        this.showOriginPx = this.showLayout.x;
        this.moveToIndex(this.selectIndex - 1);
    },
    rightMove: function () {
        if (this.isMoveing) return;
        this.showOriginPx = this.showLayout.x;
        this.moveToIndex(this.selectIndex + 1);
    },
    getSelectItem: function () {
        return this.itemList[this.centerIndex];
    },
    moveScale: function (dx) {
        var item = this.getSelectItem();
        var dScale = Math.abs(dx / this.itemSize.width * this.dScale);
        this.setItemScale(item, -dScale);

        var nItem = null;
        if (dx >= 0) nItem = this.itemList[this.centerIndex - 1];
        else nItem = this.itemList[this.centerIndex + 1];
        this.setItemScale(nItem, dScale);
    },
    setItemScale: function (item, dScale) {
        if (!item) return;
        var tagScale = item.getScale() + dScale;
        if (tagScale < this.normalScale || tagScale > this.selectScale) return;
        item.setScale(tagScale);
    },
    setMoveCallFunc: function (func) {
        this.moveCallFunc = func;
    }
});

